module MR; end
module MR::AfterCommit

  module RecordProcsMethods

    VALID_CALLBACK_TYPES = [
      :create,
      :update,
      :save,
      :destroy
    ].freeze

    DEFAULT_CALLBACK_TYPE = :save.freeze

    # these methods are used by the `Record` and `FakeRecord` mixins

    def after_commit_procs(*keys)
      if keys.empty?
        mr_after_commit_procs_hash.values.flatten
      else
        keys.map{ |k| mr_after_commit_procs_hash[k.to_sym] }.flatten
      end
    end

    def add_after_commit_proc(callback_type = nil, &block)
      callback_type ||= DEFAULT_CALLBACK_TYPE
      mr_after_commit_procs_hash[callback_type.to_sym] << block
    end

    def prepend_after_commit_proc(callback_type = nil, &block)
      callback_type ||= DEFAULT_CALLBACK_TYPE
      mr_after_commit_procs_hash[callback_type.to_sym].unshift(block)
    end

    def clear_after_commit_procs(*keys)
      if keys.empty?
        mr_after_commit_procs_hash.clear
      else
        keys.map{ |k| mr_after_commit_procs_hash.delete(k.to_sym) }
      end
    end

    def called_after_commit_procs(*keys)
      if keys.empty?
        mr_after_commit_called_procs_hash.values.flatten
      else
        keys.map{ |k| mr_after_commit_called_procs_hash[k.to_sym] }.flatten
      end
    end

    private

    def mr_after_commit_call_procs(callback_type)
      procs = self.after_commit_procs(callback_type)
      procs.each(&:call)
      mr_after_commit_called_procs_hash[callback_type] += procs
      self.clear_after_commit_procs(callback_type)
    end

    def mr_after_commit_procs_hash
      @mr_after_commit_procs_hash ||= MR::AfterCommit::CallbackProcsHash.new
    end

    def mr_after_commit_called_procs_hash
      @mr_after_commit_called_procs_hash ||= MR::AfterCommit::CallbackProcsHash.new
    end

  end

  module CallbackProcsHash
    def self.new
      Hash.new do |h, k|
        if !VALID_CALLBACK_TYPES.include?(k)
          raise ArgumentError, "#{k.inspect} is not a valid callback " \
                               "type, use: #{VALID_CALLBACK_TYPES.join(', ')}"
        end
        h[k] = []
      end
    end
  end

end
