require 'much-plugin'
require 'mr/read_model/data'
require 'mr/read_model/fields'
require 'mr/read_model/querying'

module MR

  module ReadModel
    include MuchPlugin

    plugin_included do
      include MR::ReadModelStruct
      include MR::ReadModel::Querying
      extend ClassMethods
    end

    def self.add_select_for_field(read_model_class, name, column_sql = nil, &column_sql_block)
      if column_sql
        read_model_class.select("#{column_sql} AS #{name}")
      elsif column_sql_block
        read_model_class.select do |params|
          "#{column_sql_block.call(params)} AS #{name}"
        end
      end
    end

    module ClassMethods

      def field(name, type, column_sql = nil, &column_sql_block)
        super(name, type)
        MR::ReadModel.add_select_for_field(self, name, column_sql, &column_sql_block)
      end

      def json_struct_list(name, struct_class, column_sql = nil, &column_sql_block)
        super(name, struct_class)
        MR::ReadModel.add_select_for_field(self, name, column_sql, &column_sql_block)
      end

      def json_struct_obj(name, struct_class, column_sql = nil, &column_sql_block)
        super(name, struct_class)
        MR::ReadModel.add_select_for_field(self, name, column_sql, &column_sql_block)
      end

    end

  end

  module ReadModelStruct
    include MuchPlugin

    plugin_included do
      include MR::ReadModel::Data
      include MR::ReadModel::Fields
      include InstanceMethods
    end

    module InstanceMethods

      def initialize(data = nil)
        set_read_model_data(data || {})
      end

      def ==(other)
        if other.kind_of?(self.class)
          self.fields == other.fields
        else
          super
        end
      end

      def inspect
        object_hex     = (self.object_id << 1).to_s(16)
        fields_inspect = self.class.fields.map do |field|
          "#{field.ivar_name}=#{field.read(self.read_model_data).inspect}"
        end.sort.join(" ")
        "#<#{self.class}:0x#{object_hex} #{fields_inspect}>"
      end

    end

  end

end
