require 'much-plugin'
require 'mr/read_model/querying'

module MR::ReadModel

  module SetQuerying
    include MuchPlugin

    plugin_included do
      include MR::ReadModel::Querying
      extend ClassMethods
    end

    module ClassMethods

      def relation
        @relation ||= Relation.new
      end

      def union(&block)
        add_set_expression(:union, &block)
      rescue ArgumentError => exception
        raise ArgumentError, exception.message, caller
      end

      def union_all(&block)
        add_set_expression(:union_all, &block)
      rescue ArgumentError => exception
        raise ArgumentError, exception.message, caller
      end

      def intersect(&block)
        add_set_expression(:intersect, &block)
      rescue ArgumentError => exception
        raise ArgumentError, exception.message, caller
      end

      def intersect_all(&block)
        add_set_expression(:intersect_all, &block)
      rescue ArgumentError => exception
        raise ArgumentError, exception.message, caller
      end

      def except(&block)
        add_set_expression(:except, &block)
      rescue ArgumentError => exception
        raise ArgumentError, exception.message, caller
      end

      def except_all(&block)
        add_set_expression(:except_all, &block)
      rescue ArgumentError => exception
        raise ArgumentError, exception.message, caller
      end

      private

      def add_set_expression(type, &block)
        SetExpression.new(type, &block).tap do |expression|
          relation.set_expressions << expression
        end
      end

    end

    class Relation < MR::ReadModel::Relation
      attr_reader :set_expressions

      def initialize
        super
        @set_expressions = []
      end

      def build_sql(params = nil)
        sql = super(params)
        self.set_expressions.inject(sql){ |s, e| e.combine_sql(s, params) }
      end
    end

  end

end
