module MR; end
module MR::ReadModel

  module Subquery
    attr_reader :read_model_class

    def initialize(&block)
      self.instance_eval(&block)
      if !self.read_model_class
        raise ArgumentError, "invalid subquery - " \
                             "use `read_model` to define the subquery"
      end
    end

    def read_model(&block)
      @read_model_class = Class.new do
         # TODO - fix circular require
        require 'mr/read_model'
        require 'mr/read_model/set_querying'
        include MR::ReadModel
        include MR::ReadModel::SetQuerying
      end
      @read_model_class.class_eval(&block)
    end

    def build_sql(params = nil)
      "(#{self.read_model_class.build_sql(params)})"
    end

  end

  module AliasSubquery
    include Subquery

    def alias_sql
      @alias_sql ||= ""
    end

    def as(alias_name)
      if alias_name.to_s.strip.empty?
        raise ArgumentError, "alias can't be blank"
      end
      @alias_sql = "AS #{alias_name}"
    end

    def build_sql(params = nil)
      if self.alias_sql.to_s.strip.empty?
        raise InvalidSubqueryError, "subquery must have an alias"
      end
      "#{super} #{self.alias_sql}".strip
    end

  end

  class FromSubquery
    include AliasSubquery

    def record_class
      self.read_model_class.record_class
    end

  end

  class JoinSubquery
    include AliasSubquery

    DEFAULT_JOIN_SQL = 'JOIN'.freeze
    JOIN_SQL = Hash.new(DEFAULT_JOIN_SQL).tap do |h|
      h[:inner] = 'INNER JOIN'.freeze
      h[:left]  = 'LEFT OUTER JOIN'.freeze
      h[:right] = 'RIGHT OUTER JOIN'.freeze
      h[:full]  = 'FULL OUTER JOIN'.freeze
    end.freeze

    attr_reader :join_sql

    def initialize(type, &block)
      @join_sql = JOIN_SQL[type]
      super(&block)
    end

    def conditions_sql
      @conditions_sql ||= ""
    end

    def on(conditions)
      @conditions_sql = "ON #{conditions}"
    end

    def build_sql(params = nil)
      "#{self.join_sql} #{super} #{self.conditions_sql}".strip
    end

  end

  InvalidSubqueryError = Class.new(RuntimeError)

end
